<?php

declare(strict_types=1);

namespace Drupal\rdf_sync\Encoder;

use Drupal\rdf_sync\Model\RdfSyncFormat;
use EasyRdf\Format;
use EasyRdf\Graph;
use Symfony\Component\Serializer\Encoder\EncoderInterface;

/**
 * Base encoder.
 */
class RdfSyncEncoder implements EncoderInterface {

  /**
   * Memory cache for supported formats.
   *
   * @var \EasyRdf\Serialiser[]
   */
  protected static array $supportedFormats;

  /**
   * {@inheritdoc}
   */
  public function supportsEncoding($format): bool {
    return in_array($format, static::getSupportedFormats(), TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function encode($data, $format, array $context = []): string {
    $graph = new Graph();
    $graph->parse($data, 'php');
    return $graph->serialise($format);
  }

  /**
   * Builds a list of supported formats.
   *
   * @return \EasyRdf\Serialiser[]
   *   List of supported formats.
   */
  public static function getSupportedFormats(): array {
    if (!isset(static::$supportedFormats)) {
      $registeredFormats = array_map(
        fn(RdfSyncFormat $format): string => $format->value,
        RdfSyncFormat::cases(),
      );
      $rdfSerializers = array_keys(Format::getFormats());
      static::$supportedFormats = array_intersect($rdfSerializers, $registeredFormats);
    }
    return static::$supportedFormats;
  }

}
